# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from twisted.internet import defer
from twisted.trial import unittest

from buildbot.reporters.zulip import ZulipStatusPush
from buildbot.test.fake import fakemaster
from buildbot.test.fake import httpclientservice as fakehttpclientservice
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.util.config import ConfigErrorsMixin
from buildbot.test.util.logging import LoggingMixin
from buildbot.test.util.reporter import ReporterTestMixin


class TestZulipStatusPush(unittest.TestCase, ReporterTestMixin, LoggingMixin, ConfigErrorsMixin,
                          TestReactorMixin):

    def setUp(self):
        self.setup_test_reactor()
        self.setup_reporter_test()
        self.master = fakemaster.make_master(
            testcase=self, wantData=True, wantDb=True, wantMq=True)

    @defer.inlineCallbacks
    def tearDown(self):
        if self.master.running:
            yield self.master.stopService()

    @defer.inlineCallbacks
    def setupZulipStatusPush(self, endpoint="http://example.com", token="123", stream=None):
        self.sp = ZulipStatusPush(
            endpoint=endpoint, token=token, stream=stream)
        self._http = yield fakehttpclientservice.HTTPClientService.getService(
            self.master, self, endpoint, debug=None, verify=None)
        yield self.sp.setServiceParent(self.master)
        yield self.master.startService()

    @defer.inlineCallbacks
    def test_build_started(self):
        yield self.setupZulipStatusPush(stream="xyz")
        build = yield self.insert_build_new()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123&stream=xyz',
            json={
                "event": 'new',
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000001
            })
        yield self.sp._got_event(('builds', 20, 'new'), build)

    @defer.inlineCallbacks
    def test_build_finished(self):
        yield self.setupZulipStatusPush(stream="xyz")
        build = yield self.insert_build_finished()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123&stream=xyz',
            json={
                "event": "finished",
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000005,
                "results": 0
            })
        yield self.sp._got_event(('builds', 20, 'finished'), build)

    @defer.inlineCallbacks
    def test_stream_none(self):
        yield self.setupZulipStatusPush(stream=None)
        build = yield self.insert_build_finished()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123',
            json={
                "event": "finished",
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000005,
                "results": 0
            })
        yield self.sp._got_event(('builds', 20, 'finished'), build)

    def test_endpoint_string(self):
        with self.assertRaisesConfigError(
                "Endpoint must be a string"):
            ZulipStatusPush(endpoint=1234, token="abcd")

    def test_token_string(self):
        with self.assertRaisesConfigError(
                "Token must be a string"):
            ZulipStatusPush(endpoint="http://example.com", token=1234)

    @defer.inlineCallbacks
    def test_invalid_json_data(self):
        yield self.setupZulipStatusPush(stream="xyz")
        build = yield self.insert_build_new()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123&stream=xyz',
            json={
                "event": 'new',
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000001
            }, code=500)
        self.setUpLogging()
        yield self.sp._got_event(('builds', 20, 'new'), build)
        self.assertLogged('500: Error pushing build status to Zulip')

    @defer.inlineCallbacks
    def test_invalid_url(self):
        yield self.setupZulipStatusPush(stream="xyz")
        build = yield self.insert_build_new()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123&stream=xyz',
            json={
                "event": 'new',
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000001
            }, code=404)
        self.setUpLogging()
        yield self.sp._got_event(('builds', 20, 'new'), build)
        self.assertLogged('404: Error pushing build status to Zulip')

    @defer.inlineCallbacks
    def test_invalid_token(self):
        yield self.setupZulipStatusPush(stream="xyz")
        build = yield self.insert_build_new()
        self._http.expect(
            'post',
            '/api/v1/external/buildbot?api_key=123&stream=xyz',
            json={
                "event": 'new',
                "buildid": 20,
                "buildername": "Builder0",
                "url": "http://localhost:8080/#builders/79/builds/0",
                "project": "testProject",
                "timestamp": 10000001
            }, code=401, content_json={"result": "error", "msg": "Invalid API key",
                                       "code": "INVALID_API_KEY"})
        self.setUpLogging()
        yield self.sp._got_event(('builds', 20, 'new'), build)
        self.assertLogged('401: Error pushing build status to Zulip')



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


import mock

from twisted.internet import defer
from twisted.trial import unittest

from buildbot.changes import base
from buildbot.changes import manager
from buildbot.test.fake import fakemaster
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.util.warnings import assertProducesWarnings
from buildbot.warnings import DeprecatedApiWarning


class TestChangeManager(unittest.TestCase, TestReactorMixin):

    @defer.inlineCallbacks
    def setUp(self):
        self.setup_test_reactor()
        self.master = fakemaster.make_master(self, wantData=True)
        self.cm = manager.ChangeManager()
        self.master.startService()
        yield self.cm.setServiceParent(self.master)
        self.new_config = mock.Mock()

    def tearDown(self):
        return self.master.stopService()

    def make_sources(self, n, klass=base.ChangeSource, **kwargs):
        for i in range(n):
            src = klass(name=f'ChangeSource {i}', **kwargs)
            yield src

    @defer.inlineCallbacks
    def test_reconfigService_add(self):
        src1, src2 = self.make_sources(2)
        yield src1.setServiceParent(self.cm)
        self.new_config.change_sources = [src1, src2]

        yield self.cm.reconfigServiceWithBuildbotConfig(self.new_config)

        self.assertIdentical(src2.parent, self.cm)
        self.assertIdentical(src2.master, self.master)

    @defer.inlineCallbacks
    def test_reconfigService_remove(self):
        src1, = self.make_sources(1)
        yield src1.setServiceParent(self.cm)
        self.new_config.change_sources = []

        self.assertTrue(src1.running)
        yield self.cm.reconfigServiceWithBuildbotConfig(self.new_config)

        self.assertFalse(src1.running)

    @defer.inlineCallbacks
    def test_reconfigService_change_reconfigurable(self):
        src1, = self.make_sources(1, base.ReconfigurablePollingChangeSource, pollInterval=1)
        yield src1.setServiceParent(self.cm)

        src2, = self.make_sources(1, base.ReconfigurablePollingChangeSource, pollInterval=2)

        self.new_config.change_sources = [src2]

        self.assertTrue(src1.running)
        self.assertEqual(src1.pollInterval, 1)
        yield self.cm.reconfigServiceWithBuildbotConfig(self.new_config)

        self.assertTrue(src1.running)
        self.assertFalse(src2.running)
        self.assertEqual(src1.pollInterval, 2)

    @defer.inlineCallbacks
    def test_reconfigService_change_legacy(self):
        with assertProducesWarnings(DeprecatedApiWarning,
                                    message_pattern="use ReconfigurablePollingChangeSource"):
            src1, = self.make_sources(1, base.PollingChangeSource, pollInterval=1)
        yield src1.setServiceParent(self.cm)

        with assertProducesWarnings(DeprecatedApiWarning,
                                    message_pattern="use ReconfigurablePollingChangeSource"):
            src2, = self.make_sources(1, base.PollingChangeSource, pollInterval=2)

        self.new_config.change_sources = [src2]

        self.assertTrue(src1.running)
        self.assertEqual(src1.pollInterval, 1)
        yield self.cm.reconfigServiceWithBuildbotConfig(self.new_config)

        self.assertFalse(src1.running)
        self.assertTrue(src2.running)
        self.assertEqual(src2.pollInterval, 2)



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

import os

import mock

from twisted.internet import defer
from twisted.trial import unittest

from buildbot.db import connector
from buildbot.db import model
from buildbot.scripts import create_master
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.util import dirs
from buildbot.test.util import misc
from buildbot.test.util import www


def mkconfig(**kwargs):
    config = dict(force=False, relocatable=False, config='master.cfg',
                  db='sqlite:///state.sqlite', basedir=os.path.abspath('basedir'),
                  quiet=False, **{'no-logrotate': False, 'log-size': 10000000,
                                  'log-count': 10})
    config.update(kwargs)
    return config


class TestCreateMaster(misc.StdoutAssertionsMixin, unittest.TestCase):

    def setUp(self):
        # createMaster is decorated with @in_reactor, so strip that decoration
        # since the master is already running
        self.patch(create_master, 'createMaster',
                   create_master.createMaster._orig)
        self.setUpStdoutAssertions()

    # tests

    @defer.inlineCallbacks
    def do_test_createMaster(self, config):
        # mock out everything that createMaster calls, then check that
        # they are called, in order
        functions = ['makeBasedir', 'makeTAC', 'makeSampleConfig',
                     'createDB']
        repls = {}
        calls = []
        for fn in functions:
            repl = repls[fn] = mock.Mock(name=fn)
            repl.side_effect = lambda config, fn=fn: calls.append(fn)
            self.patch(create_master, fn, repl)
        repls['createDB'].side_effect = (lambda config:
                                         calls.append(fn) or defer.succeed(None))
        rc = yield create_master.createMaster(config)

        self.assertEqual(rc, 0)
        self.assertEqual(calls, functions)
        for repl in repls.values():
            repl.assert_called_with(config)

    @defer.inlineCallbacks
    def test_createMaster_quiet(self):
        yield self.do_test_createMaster(mkconfig(quiet=True))

        self.assertWasQuiet()

    @defer.inlineCallbacks
    def test_createMaster_loud(self):
        yield self.do_test_createMaster(mkconfig(quiet=False))

        self.assertInStdout('buildmaster configured in')


class TestCreateMasterFunctions(www.WwwTestMixin, dirs.DirsMixin,
                                misc.StdoutAssertionsMixin, TestReactorMixin,
                                unittest.TestCase):

    def setUp(self):
        self.setup_test_reactor()
        self.setUpDirs('test')
        self.basedir = os.path.abspath(os.path.join('test', 'basedir'))
        self.setUpStdoutAssertions()

    def tearDown(self):
        self.tearDownDirs()

    def assertInTacFile(self, str):
        with open(os.path.join('test', 'buildbot.tac'), 'rt', encoding='utf-8') as f:
            content = f.read()
        self.assertIn(str, content)

    def assertNotInTacFile(self, str):
        with open(os.path.join('test', 'buildbot.tac'), 'rt', encoding='utf-8') as f:
            content = f.read()
        self.assertNotIn(str, content)

    def assertDBSetup(self, basedir=None, db_url='sqlite:///state.sqlite',
                      verbose=True):
        # mock out the database setup
        self.db = mock.Mock()
        self.db.setup.side_effect = lambda *a, **k: defer.succeed(None)
        self.DBConnector = mock.Mock()
        self.DBConnector.return_value = self.db
        self.patch(connector, 'DBConnector', self.DBConnector)

        basedir = basedir or self.basedir
        # pylint: disable=unsubscriptable-object
        self.assertEqual(
            dict(basedir=self.DBConnector.call_args[0][1],
                 db_url=self.DBConnector.call_args[0][0].mkconfig.db['db_url'],
                 verbose=self.db.setup.call_args[1]['verbose'],
                 check_version=self.db.setup.call_args[1]['check_version'],
                 ),
            dict(basedir=self.basedir,
                 db_url=db_url,
                 verbose=True,
                 check_version=False))

    # tests

    def test_makeBasedir(self):
        self.assertFalse(os.path.exists(self.basedir))
        create_master.makeBasedir(mkconfig(basedir=self.basedir))
        self.assertTrue(os.path.exists(self.basedir))
        self.assertInStdout(f'mkdir {self.basedir}')

    def test_makeBasedir_quiet(self):
        self.assertFalse(os.path.exists(self.basedir))
        create_master.makeBasedir(mkconfig(basedir=self.basedir, quiet=True))
        self.assertTrue(os.path.exists(self.basedir))
        self.assertWasQuiet()

    def test_makeBasedir_existing(self):
        os.mkdir(self.basedir)
        create_master.makeBasedir(mkconfig(basedir=self.basedir))
        self.assertInStdout('updating existing installation')

    def test_makeTAC(self):
        create_master.makeTAC(mkconfig(basedir='test'))
        self.assertInTacFile("Application('buildmaster')")
        self.assertWasQuiet()

    def test_makeTAC_relocatable(self):
        create_master.makeTAC(mkconfig(basedir='test', relocatable=True))
        self.assertInTacFile("basedir = '.'")  # repr() prefers ''
        self.assertWasQuiet()

    def test_makeTAC_no_logrotate(self):
        create_master.makeTAC(
            mkconfig(basedir='test', **{'no-logrotate': True}))
        self.assertNotInTacFile("import Log")
        self.assertWasQuiet()

    def test_makeTAC_int_log_count(self):
        create_master.makeTAC(mkconfig(basedir='test', **{'log-count': 30}))
        self.assertInTacFile("\nmaxRotatedFiles = 30\n")
        self.assertWasQuiet()

    def test_makeTAC_str_log_count(self):
        with self.assertRaises(TypeError):
            create_master.makeTAC(mkconfig(basedir='test',
                                  **{'log-count': '30'}))

    def test_makeTAC_none_log_count(self):
        create_master.makeTAC(mkconfig(basedir='test', **{'log-count': None}))
        self.assertInTacFile("\nmaxRotatedFiles = None\n")
        self.assertWasQuiet()

    def test_makeTAC_int_log_size(self):
        create_master.makeTAC(mkconfig(basedir='test', **{'log-size': 3000}))
        self.assertInTacFile("\nrotateLength = 3000\n")
        self.assertWasQuiet()

    def test_makeTAC_str_log_size(self):
        with self.assertRaises(TypeError):
            create_master.makeTAC(mkconfig(basedir='test',
                                  **{'log-size': '3000'}))

    def test_makeTAC_existing_incorrect(self):
        with open(os.path.join('test', 'buildbot.tac'), 'wt', encoding='utf-8') as f:
            f.write('WRONG')
        create_master.makeTAC(mkconfig(basedir='test'))
        self.assertInTacFile("WRONG")
        self.assertTrue(os.path.exists(
            os.path.join('test', 'buildbot.tac.new')))
        self.assertInStdout('not touching existing buildbot.tac')

    def test_makeTAC_existing_incorrect_quiet(self):
        with open(os.path.join('test', 'buildbot.tac'), 'wt', encoding='utf-8') as f:
            f.write('WRONG')
        create_master.makeTAC(mkconfig(basedir='test', quiet=True))
        self.assertInTacFile("WRONG")
        self.assertWasQuiet()

    def test_makeTAC_existing_correct(self):
        create_master.makeTAC(mkconfig(basedir='test', quiet=True))
        create_master.makeTAC(mkconfig(basedir='test'))
        self.assertFalse(os.path.exists(
            os.path.join('test', 'buildbot.tac.new')))
        self.assertInStdout('and is correct')

    def test_makeSampleConfig(self):
        create_master.makeSampleConfig(mkconfig(basedir='test'))
        self.assertTrue(os.path.exists(
            os.path.join('test', 'master.cfg.sample')))
        self.assertInStdout('creating ')

    def test_makeSampleConfig_db(self):
        create_master.makeSampleConfig(mkconfig(basedir='test', db='XXYYZZ',
                                                quiet=True))
        with open(os.path.join('test', 'master.cfg.sample'), 'rt', encoding='utf-8') as f:
            self.assertIn("XXYYZZ", f.read())
        self.assertWasQuiet()

    @defer.inlineCallbacks
    def test_createDB(self):
        setup = mock.Mock(side_effect=lambda **kwargs: defer.succeed(None))
        self.patch(connector.DBConnector, 'setup', setup)
        upgrade = mock.Mock(side_effect=lambda **kwargs: defer.succeed(None))
        self.patch(model.Model, 'upgrade', upgrade)
        yield create_master.createDB(
            mkconfig(basedir='test', quiet=True),
            _noMonkey=True)
        setup.asset_called_with(check_version=False, verbose=False)
        upgrade.assert_called_with()
        self.assertWasQuiet()



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


import json
import os

from twisted.internet import defer
from twisted.trial import unittest

from buildbot.data import connector as dataconnector
from buildbot.data.graphql import GraphQLConnector
from buildbot.mq import connector as mqconnector
from buildbot.process.results import SUCCESS
from buildbot.schedulers.forcesched import ForceScheduler
from buildbot.test import fakedb
from buildbot.test.fake import fakemaster
from buildbot.test.reactor import TestReactorMixin
from buildbot.util import toJson

try:
    from ruamel.yaml import YAML
except ImportError:
    YAML = None


try:
    import graphql as graphql_core
except ImportError:
    graphql_core = None


class GraphQL(unittest.TestCase, TestReactorMixin):
    if not graphql_core:
        skip = "graphql-core is required for GraphQL integration tests"

    master = None

    def load_yaml(self, f):
        if YAML is None:
            # for running the test ruamel is not needed (to avoid a build dependency for distros)
            import yaml
            return yaml.safe_load(f)
        self.yaml = YAML()
        self.yaml.default_flow_style = False
        # default is round-trip
        return self.yaml.load(f)

    def save_yaml(self, data, f):
        if YAML is None:
            raise ImportError("please install ruamel.yaml for test regeneration")
        self.yaml.dump(data, f)

    @defer.inlineCallbacks
    def setUp(self):
        self.setup_test_reactor(use_asyncio=True)

        master = fakemaster.make_master(self)
        master.db = fakedb.FakeDBConnector(self)
        yield master.db.setServiceParent(master)

        master.config.mq = {'type': "simple"}
        master.mq = mqconnector.MQConnector()
        yield master.mq.setServiceParent(master)
        yield master.mq.setup()

        master.data = dataconnector.DataConnector()
        yield master.data.setServiceParent(master)

        master.graphql = GraphQLConnector()
        yield master.graphql.setServiceParent(master)

        master.config.www = {'graphql': {"debug": True}}
        master.graphql.reconfigServiceWithBuildbotConfig(master.config)

        self.master = master
        scheds = [ForceScheduler(
            name="force",
            builderNames=["runtests0", "runtests1", "runtests2", "slowruntests"])]
        self.master.allSchedulers = lambda: scheds

        yield self.master.startService()

        yield self.insert_initial_data()

    @defer.inlineCallbacks
    def tearDown(self):
        yield self.master.stopService()

    def insert_initial_data(self):
        self.master.db.insertTestData([
            fakedb.Master(id=1),
            fakedb.Worker(id=1, name='example-worker'),

            fakedb.Scheduler(id=1, name='custom', enabled=1),
            fakedb.Scheduler(id=2, name='all', enabled=2),
            fakedb.Scheduler(id=3, name='force', enabled=3),

            fakedb.SchedulerMaster(schedulerid=1, masterid=1),
            fakedb.SchedulerMaster(schedulerid=2, masterid=1),
            fakedb.SchedulerMaster(schedulerid=3, masterid=1),

            fakedb.Builder(id=1, name='runtests1'),
            fakedb.Builder(id=2, name='runtests2'),
            fakedb.Builder(id=3, name='runtests3'),

            fakedb.BuilderMaster(id=1, builderid=1, masterid=1),
            fakedb.BuilderMaster(id=2, builderid=2, masterid=1),
            fakedb.BuilderMaster(id=3, builderid=3, masterid=1),

            fakedb.Tag(id=1, name='tag1'),
            fakedb.Tag(id=2, name='tag12'),
            fakedb.Tag(id=3, name='tag23'),

            fakedb.BuildersTags(id=1, builderid=1, tagid=1),
            fakedb.BuildersTags(id=2, builderid=1, tagid=2),
            fakedb.BuildersTags(id=3, builderid=2, tagid=2),
            fakedb.BuildersTags(id=4, builderid=2, tagid=3),
            fakedb.BuildersTags(id=5, builderid=3, tagid=3),

            fakedb.Buildset(id=1, results=SUCCESS, reason="Force reason 1",
                            submitted_at=100000, complete_at=100110, complete=1),
            fakedb.Buildset(id=2, results=SUCCESS, reason="Force reason 2",
                            submitted_at=100200, complete_at=100330, complete=1),
            fakedb.Buildset(id=3, results=SUCCESS, reason="Force reason 3",
                            submitted_at=100400, complete_at=100550, complete=1),

            fakedb.BuildsetProperty(buildsetid=1, property_name='scheduler',
                                    property_value='["custom", "Scheduler"]'),
            fakedb.BuildsetProperty(buildsetid=2, property_name='scheduler',
                                    property_value='["all", "Scheduler"]'),
            fakedb.BuildsetProperty(buildsetid=3, property_name='scheduler',
                                    property_value='["force", "Scheduler"]'),
            fakedb.BuildsetProperty(buildsetid=3, property_name='owner',
                                    property_value='["some@example.com", "Force Build Form"]'),

            fakedb.SourceStamp(id=1, branch='master', revision='1234abcd'),
            fakedb.Change(changeid=1, branch='master', revision='1234abcd', sourcestampid=1),
            fakedb.ChangeProperty(changeid=1, property_name="owner",
                                  property_value='["me@example.com", "change"]'),
            fakedb.ChangeProperty(changeid=1, property_name="other_prop",
                                  property_value='["value", "change"]'),
            fakedb.BuildsetSourceStamp(id=1, buildsetid=1, sourcestampid=1),
            fakedb.BuildsetSourceStamp(id=2, buildsetid=2, sourcestampid=1),
            fakedb.BuildsetSourceStamp(id=3, buildsetid=3, sourcestampid=1),

            fakedb.BuildRequest(id=1, buildsetid=1, builderid=1, results=SUCCESS,
                                submitted_at=100001, complete_at=100109, complete=1),
            fakedb.BuildRequest(id=2, buildsetid=2, builderid=1, results=SUCCESS,
                                submitted_at=100201, complete_at=100329, complete=1),
            fakedb.BuildRequest(id=3, buildsetid=3, builderid=2, results=SUCCESS,
                                submitted_at=100401, complete_at=100549, complete=1),

            fakedb.Build(id=1, number=1, buildrequestid=1, builderid=1, workerid=1,
                         masterid=1001, started_at=100002, complete_at=100108,
                         state_string='build successful', results=SUCCESS),
            fakedb.Build(id=2, number=2, buildrequestid=2, builderid=1, workerid=1,
                         masterid=1001, started_at=100202, complete_at=100328,
                         state_string='build successful', results=SUCCESS),
            fakedb.Build(id=3, number=1, buildrequestid=3, builderid=2, workerid=1,
                         masterid=1001, started_at=100402, complete_at=100548,
                         state_string='build successful', results=SUCCESS),

            fakedb.BuildProperty(buildid=3, name='reason', value='"force build"',
                                 source="Force Build Form"),
            fakedb.BuildProperty(buildid=3, name='owner', value='"some@example.com"',
                                 source="Force Build Form"),
            fakedb.BuildProperty(buildid=3, name='scheduler', value='"force"',
                                 source="Scheduler"),
            fakedb.BuildProperty(buildid=3, name='buildername', value='"runtests3"',
                                 source="Builder"),
            fakedb.BuildProperty(buildid=3, name='workername', value='"example-worker"',
                                 source="Worker"),

            fakedb.Step(id=1, number=1, name='step1', buildid=1,
                        started_at=100010, complete_at=100019, state_string='step1 done'),
            fakedb.Step(id=2, number=2, name='step2', buildid=1,
                        started_at=100020, complete_at=100029, state_string='step2 done'),
            fakedb.Step(id=3, number=3, name='step3', buildid=1,
                        started_at=100030, complete_at=100039, state_string='step3 done'),
            fakedb.Step(id=11, number=1, name='step1', buildid=2,
                        started_at=100210, complete_at=100219, state_string='step1 done'),
            fakedb.Step(id=12, number=2, name='step2', buildid=2,
                        started_at=100220, complete_at=100229, state_string='step2 done'),
            fakedb.Step(id=13, number=3, name='step3', buildid=2,
                        started_at=100230, complete_at=100239, state_string='step3 done'),
            fakedb.Step(id=21, number=1, name='step1', buildid=3,
                        started_at=100410, complete_at=100419, state_string='step1 done'),
            fakedb.Step(id=22, number=2, name='step2', buildid=3,
                        started_at=100420, complete_at=100429, state_string='step2 done'),
            fakedb.Step(id=23, number=3, name='step3', buildid=3,
                        started_at=100430, complete_at=100439, state_string='step3 done'),

            fakedb.Log(id=1, name='stdio', slug='stdio', stepid=1, complete=1, num_lines=10),
            fakedb.Log(id=2, name='stdio', slug='stdio', stepid=2, complete=1, num_lines=20),
            fakedb.Log(id=3, name='stdio', slug='stdio', stepid=3, complete=1, num_lines=30),
            fakedb.Log(id=11, name='stdio', slug='stdio', stepid=11, complete=1, num_lines=30),
            fakedb.Log(id=12, name='stdio', slug='stdio', stepid=12, complete=1, num_lines=40),
            fakedb.Log(id=13, name='stdio', slug='stdio', stepid=13, complete=1, num_lines=50),
            fakedb.Log(id=21, name='stdio', slug='stdio', stepid=21, complete=1, num_lines=50),
            fakedb.Log(id=22, name='stdio', slug='stdio', stepid=22, complete=1, num_lines=60),
            fakedb.Log(id=23, name='stdio', slug='stdio', stepid=23, complete=1, num_lines=70),

            fakedb.LogChunk(logid=1, first_line=0, last_line=2,
                            content='o line1\no line2\n'),
            fakedb.LogChunk(logid=1, first_line=2, last_line=3,
                            content='o line3\n'),
            fakedb.LogChunk(logid=2, first_line=0, last_line=4,
                            content='o line1\no line2\no line3\no line4\n'),
        ])

    @defer.inlineCallbacks
    def test_examples_from_yaml(self):
        """This test takes input from yaml file containing queries to execute and
        expected results. In order to ease writing of tests, if the expected key is not found,
        it is automatically generated, so developer only has to review results
        Full regen can still be done with regen local variable just below
        """
        regen = False
        need_save = False
        fn = os.path.join(os.path.dirname(__file__), "test_graphql_queries.yaml")
        with open(fn, encoding='utf-8') as f:
            data = self.load_yaml(f)
        focussed_data = [test for test in data if test.get('focus')]
        if not focussed_data:
            focussed_data = data
        for test in focussed_data:
            query = test['query']
            result = yield self.master.graphql.query(
                query
            )
            self.assertIsNone(result.errors)
            if 'expected' not in test or regen:
                need_save = True
                test['expected'] = result.data
            else:
                # remove ruamel metadata before compare (it is needed for round-trip regen,
                # but confuses the comparison)
                result_data = json.loads(json.dumps(result.data, default=toJson))
                expected = json.loads(json.dumps(test['expected'], default=toJson))
                self.assertEqual(
                    result_data, expected, f"for {query}")
        if need_save:
            with open(fn, 'w', encoding='utf-8') as f:
                self.save_yaml(data, f)

    @defer.inlineCallbacks
    def test_buildrequests_builds(self):
        data = yield self.master.graphql.query(
            "{buildrequests{buildrequestid, builds{number, buildrequestid}}}"
        )

        self.assertEqual(data.errors, None)
        for br in data.data["buildrequests"]:
            for build in br["builds"]:
                self.assertEqual(build["buildrequestid"], br["buildrequestid"])



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from subprocess import CalledProcessError
from subprocess import check_output

import requests
import yaml

s = requests.Session()
with open(os.path.expanduser('~/.config/hub')) as f:
    config = yaml.load(f)['github.com'][0]
    s.auth = config['user'], config['oauth_token']

os.system("git fetch --all")
r = s.get("https://api.github.com/search/issues?q=label:\"port%20to%20stable\"+repo:buildbot/buildbot") # noqa pylint: disable=line-too-long
to_port = r.json()
summary = ""
for pr in to_port['items']:
    r = s.get("https://api.github.com/repos/buildbot/buildbot/pulls/{number}/commits".format(**pr))
    commits = r.json()
    for c in commits:
        title = c['commit']['message'].split("\n")[0]
        try:
            check_output("git cherry-pick {sha} 2>&1".format(**c), shell=True)
        except CalledProcessError as e:
            os.system("git diff")
            os.system("git reset --hard HEAD 2>&1 >/dev/null")
            if '--allow-empty' in e.output:
                continue
            if 'fatal: bad object' in e.output:
                continue
            print("cannot automatically cherry-pick", pr['number'], c['sha'], title, e.output)
        else:
            summary += "\n#{number}: {title}".format(number=pr['number'], title=title, **c)
print(summary)



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members


from twisted.internet import defer

from buildbot.test.util.integration import RunMasterBase


class ShellMaster(RunMasterBase):

    @defer.inlineCallbacks
    def test_shell(self):
        yield self.setupConfig(masterConfig())

        @defer.inlineCallbacks
        def newStepCallback(_, data):
            # when the sleep step start, we kill it
            if data['name'] == 'sleep':
                brs = yield self.master.data.get(('buildrequests',))
                brid = brs[-1]['buildrequestid']
                self.master.data.control(
                    'cancel', {'reason': 'cancelled by test'}, ('buildrequests', brid))

        yield self.master.mq.startConsuming(
            newStepCallback,
            ('steps', None, 'new'))

        build = yield self.doForceBuild(wantSteps=True, wantLogs=True, wantProperties=True)
        self.assertEqual(build['buildid'], 1)

        # make sure the cancel reason is transferred all the way to the step log
        cancel_log = build['steps'][1]['logs'][-1]
        self.assertEqual(cancel_log['name'], 'cancelled')
        self.assertIn('cancelled by test', cancel_log['contents']['content'])


# master configuration
def masterConfig():
    c = {}
    from buildbot.config import BuilderConfig
    from buildbot.process.factory import BuildFactory
    from buildbot.plugins import steps, schedulers

    c['schedulers'] = [
        schedulers.AnyBranchScheduler(
            name="sched",
            builderNames=["testy"]),
        schedulers.ForceScheduler(
            name="force",
            builderNames=["testy"])]

    f = BuildFactory()
    f.addStep(steps.ShellCommand(command='sleep 100', name='sleep'))
    c['builders'] = [
        BuilderConfig(name="testy",
                      workernames=["local1"],
                      factory=f)]
    return c



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from twisted.internet import defer
from twisted.trial import unittest

from buildbot.process.properties import WithProperties
from buildbot.process.results import EXCEPTION
from buildbot.process.results import FAILURE
from buildbot.process.results import SUCCESS
from buildbot.process.results import WARNINGS
from buildbot.steps import shellsequence
from buildbot.test.reactor import TestReactorMixin
from buildbot.test.steps import ExpectShell
from buildbot.test.steps import TestBuildStepMixin
from buildbot.test.util import config as configmixin
from buildbot.test.util.warnings import assertProducesWarnings
from buildbot.warnings import DeprecatedApiWarning


class DynamicRun(shellsequence.ShellSequence):

    def run(self):
        return self.runShellSequence(self.dynamicCommands)


class TestOneShellCommand(TestBuildStepMixin, configmixin.ConfigErrorsMixin,
                          TestReactorMixin, unittest.TestCase):

    def setUp(self):
        self.setup_test_reactor()
        return self.setup_test_build_step()

    def tearDown(self):
        return self.tear_down_test_build_step()

    def test_shell_arg_warn_deprecated_logfile(self):
        with assertProducesWarnings(DeprecatedApiWarning,
                                    message_pattern="logfile is deprecated, use logname"):
            shellsequence.ShellArg(command="command", logfile="logfile")

    def test_shell_arg_error_logfile_and_logname(self):
        with assertProducesWarnings(DeprecatedApiWarning,
                                    message_pattern="logfile is deprecated, use logname"):
            with self.assertRaisesConfigError(
                    "the 'logfile' parameter must not be specified when 'logname' is set"):
                shellsequence.ShellArg(command="command", logname="logname", logfile="logfile")

    def testShellArgInput(self):
        with self.assertRaisesConfigError(
             "the 'command' parameter of ShellArg must not be None"):
            shellsequence.ShellArg(command=None)
        arg1 = shellsequence.ShellArg(command=1)
        with self.assertRaisesConfigError(
                "1 is an invalid command, it must be a string or a list"):
            arg1.validateAttributes()
        arg2 = shellsequence.ShellArg(command=["make", 1])
        with self.assertRaisesConfigError(
                "['make', 1] must only have strings in it"):
            arg2.validateAttributes()

        for goodcmd in ["make p1", ["make", "p1"]]:
            arg = shellsequence.ShellArg(command=goodcmd)
            arg.validateAttributes()

    def testShellArgsAreRendered(self):
        arg1 = shellsequence.ShellArg(command=WithProperties('make %s', 'project'))
        self.setup_step(
            shellsequence.ShellSequence(commands=[arg1],
                                        workdir='build'))
        self.properties.setProperty("project", "BUILDBOT-TEST", "TEST")
        self.expect_commands(ExpectShell(workdir='build', command='make BUILDBOT-TEST').exit(0))
        # TODO: need to factor command-summary stuff into a utility method and
        # use it here
        self.expect_outcome(result=SUCCESS, state_string="'make BUILDBOT-TEST'")
        return self.run_step()

    def createDynamicRun(self, commands):
        DynamicRun.dynamicCommands = commands
        return DynamicRun()

    def testSanityChecksAreDoneInRuntimeWhenDynamicCmdIsNone(self):
        self.setup_step(self.createDynamicRun(None))
        self.expect_outcome(result=EXCEPTION,
                           state_string="finished (exception)")
        return self.run_step()

    def testSanityChecksAreDoneInRuntimeWhenDynamicCmdIsString(self):
        self.setup_step(self.createDynamicRun(["one command"]))
        self.expect_outcome(result=EXCEPTION,
                           state_string='finished (exception)')
        return self.run_step()

    def testSanityChecksAreDoneInRuntimeWhenDynamicCmdIsInvalidShellArg(self):
        self.setup_step(
            self.createDynamicRun([shellsequence.ShellArg(command=1)]))
        self.expect_outcome(result=EXCEPTION,
                           state_string='finished (exception)')
        return self.run_step()

    def testMultipleCommandsAreRun(self):
        arg1 = shellsequence.ShellArg(command='make p1')
        arg2 = shellsequence.ShellArg(command='deploy p1')
        self.setup_step(
            shellsequence.ShellSequence(commands=[arg1, arg2],
                                        workdir='build'))
        self.expect_commands(ExpectShell(workdir='build', command='make p1').exit(0),
                            ExpectShell(workdir='build', command='deploy p1').exit(0))
        self.expect_outcome(result=SUCCESS, state_string="'deploy p1'")
        return self.run_step()

    def testSkipWorks(self):
        arg1 = shellsequence.ShellArg(command='make p1')
        arg2 = shellsequence.ShellArg(command='')
        arg3 = shellsequence.ShellArg(command='deploy p1')
        self.setup_step(
            shellsequence.ShellSequence(commands=[arg1, arg2, arg3],
                                        workdir='build'))
        self.expect_commands(ExpectShell(workdir='build', command='make p1').exit(0),
                            ExpectShell(workdir='build', command='deploy p1').exit(0))
        self.expect_outcome(result=SUCCESS, state_string="'deploy p1'")
        return self.run_step()

    def testWarningWins(self):
        arg1 = shellsequence.ShellArg(command='make p1',
                                      warnOnFailure=True,
                                      flunkOnFailure=False)
        arg2 = shellsequence.ShellArg(command='deploy p1')
        self.setup_step(
            shellsequence.ShellSequence(commands=[arg1, arg2],
                                        workdir='build'))
        self.expect_commands(ExpectShell(workdir='build', command='make p1').exit(1),
                            ExpectShell(workdir='build', command='deploy p1').exit(0))
        self.expect_outcome(result=WARNINGS, state_string="'deploy p1' (warnings)")
        return self.run_step()

    def testSequenceStopsOnHaltOnFailure(self):
        arg1 = shellsequence.ShellArg(command='make p1', haltOnFailure=True)
        arg2 = shellsequence.ShellArg(command='deploy p1')

        self.setup_step(
            shellsequence.ShellSequence(commands=[arg1, arg2],
                                        workdir='build'))
        self.expect_commands(ExpectShell(workdir='build', command='make p1').exit(1))
        self.expect_outcome(result=FAILURE, state_string="'make p1' (failure)")
        return self.run_step()

    @defer.inlineCallbacks
    def testShellArgsAreRenderedAnewAtEachBuild(self):
        """Unit test to ensure that ShellArg instances are properly re-rendered.

        This unit test makes sure that ShellArg instances are rendered anew at
        each new build.
        """
        arg = shellsequence.ShellArg(command=WithProperties('make %s', 'project'))
        step = shellsequence.ShellSequence(commands=[arg], workdir='build')

        # First "build"
        self.setup_step(step)
        self.properties.setProperty("project", "BUILDBOT-TEST-1", "TEST")
        self.expect_commands(ExpectShell(workdir='build', command='make BUILDBOT-TEST-1').exit(0))
        self.expect_outcome(result=SUCCESS,
                           state_string="'make BUILDBOT-TEST-1'")
        yield self.run_step()

        # Second "build"
        self.setup_step(step)
        self.properties.setProperty("project", "BUILDBOT-TEST-2", "TEST")
        self.expect_commands(ExpectShell(workdir='build', command='make BUILDBOT-TEST-2').exit(0))
        self.expect_outcome(result=SUCCESS,
                           state_string="'make BUILDBOT-TEST-2'")

        yield self.run_step()



# This file is part of Buildbot.  Buildbot is free software: you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright Buildbot Team Members

from twisted.internet import defer

from buildbot.test.fakedb.base import FakeDBComponent
from buildbot.test.fakedb.row import Row


class TestName(Row):
    table = 'test_names'

    id_column = 'id'
    foreignKeys = ('builderid',)
    required_columns = ('builderid', 'name')

    def __init__(self, id=None, builderid=None, name='nam'):
        super().__init__(id=id, builderid=builderid, name=name)


class TestCodePath(Row):
    table = 'test_code_paths'

    id_column = 'id'
    foreignKeys = ('builderid',)
    required_columns = ('builderid', 'path')

    def __init__(self, id=None, builderid=None, path='path/to/file'):
        super().__init__(id=id, builderid=builderid, path=path)


class TestResult(Row):
    table = 'test_results'

    id_column = 'id'
    foreignKeys = ('builderid', 'test_result_setid', 'test_nameid', 'test_code_pathid')
    required_columns = ('builderid', 'test_result_setid', 'value')

    def __init__(self, id=None, builderid=None, test_result_setid=None, test_nameid=None,
                 test_code_pathid=None, line=None, duration_ns=None, value=None):
        super().__init__(id=id, builderid=builderid, test_result_setid=test_result_setid,
                         test_nameid=test_nameid, test_code_pathid=test_code_pathid,
                         line=line, duration_ns=duration_ns, value=value)


class FakeTestResultsComponent(FakeDBComponent):

    def setUp(self):
        self.results = {}
        self.code_paths = {}
        self.names = {}

    def insertTestData(self, rows):
        for row in rows:
            if isinstance(row, TestName):
                self.names[row.id] = row.values.copy()

        for row in rows:
            if isinstance(row, TestCodePath):
                self.code_paths[row.id] = row.values.copy()

        for row in rows:
            if isinstance(row, TestResult):
                if row.test_nameid is not None:
                    assert row.test_nameid in self.names
                if row.test_code_pathid is not None:
                    assert row.test_code_pathid in self.code_paths

                self.results[row.id] = row.values.copy()

    def _add_code_paths(self, builderid, paths):
        path_to_id = {}

        for path in sorted(paths):
            id = self._get_code_path_id(builderid, path)
            if id is not None:
                path_to_id[path] = id
                continue

            id = Row.nextId()
            self.code_paths[id] = {
                'builderid': builderid,
                'path': path
            }
            path_to_id[path] = id

        return path_to_id

    def _get_code_path_id(self, builderid, path):
        for id, path_dict in self.code_paths.items():
            if path_dict['builderid'] == builderid and path_dict['path'] == path:
                return id
        return None

    def _add_names(self, builderid, names):
        name_to_id = {}

        for name in sorted(names):
            id = self._get_name_id(builderid, name)
            if id is not None:
                name_to_id[name] = id
                continue

            id = Row.nextId()
            self.names[id] = {
                'builderid': builderid,
                'name': name
            }
            name_to_id[name] = id

        return name_to_id

    def _get_name_id(self, builderid, name):
        for id, name_dict in self.names.items():
            if name_dict['builderid'] == builderid and name_dict['name'] == name:
                return id
        return None

    @defer.inlineCallbacks
    def addTestResults(self, builderid, test_result_setid, result_values):
        insert_code_paths = set()
        insert_names = set()
        for result_value in result_values:
            if 'value' not in result_value:
                raise KeyError('Each of result_values must contain \'value\' key')

            if 'test_name' not in result_value and 'test_code_path' not in result_value:
                raise KeyError('Each of result_values must contain at least one of '
                               '\'test_name\' or \'test_code_path\' keys')

            if 'test_name' in result_value:
                insert_names.add(result_value['test_name'])
            if 'test_code_path' in result_value:
                insert_code_paths.add(result_value['test_code_path'])

        code_path_to_id = yield self._add_code_paths(builderid, insert_code_paths)
        name_to_id = yield self._add_names(builderid, insert_names)

        for result_value in result_values:
            insert_value = {
                'value': result_value['value'],
                'builderid': builderid,
                'test_result_setid': test_result_setid,
                'test_nameid': None,
                'test_code_pathid': None,
                'duration_ns': None,
                'line': None,
            }

            if 'test_name' in result_value:
                insert_value['test_nameid'] = name_to_id[result_value['test_name']]
            if 'test_code_path' in result_value:
                insert_value['test_code_pathid'] = code_path_to_id[result_value['test_code_path']]
            if 'line' in result_value:
                insert_value['line'] = result_value['line']
            if 'duration_ns' in result_value:
                insert_value['duration_ns'] = result_value['duration_ns']

            self.results[Row.nextId()] = insert_value

    # returns a Deferred
    def getTestNames(self, builderid, name_prefix=None, result_spec=None):
        ret = []
        for _, row in sorted(self.names.items()):
            if row['builderid'] != builderid:
                continue
            if name_prefix is not None and not row['name'].startswith(name_prefix):
                continue
            ret.append(row['name'])

        if result_spec is not None:
            ret = self.applyResultSpec(ret, result_spec)

        return defer.succeed(ret)

    # returns a Deferred
    def getTestCodePaths(self, builderid, path_prefix=None, result_spec=None):
        ret = []
        for _, row in sorted(self.code_paths.items()):
            if row['builderid'] != builderid:
                continue
            if path_prefix is not None and not row['path'].startswith(path_prefix):
                continue
            ret.append(row['path'])

        if result_spec is not None:
            ret = self.applyResultSpec(ret, result_spec)

        return defer.succeed(ret)

    def _fill_extra_data(self, id, row):
        row = row.copy()
        row['id'] = id

        if row['test_nameid'] is not None:
            row['test_name'] = self.names[row['test_nameid']]['name']
        else:
            row['test_name'] = None
        del row['test_nameid']

        if row['test_code_pathid'] is not None:
            row['test_code_path'] = self.code_paths[row['test_code_pathid']]['path']
        else:
            row['test_code_path'] = None
        del row['test_code_pathid']

        return row

    # returns a Deferred
    def getTestResult(self, test_resultid):
        if test_resultid not in self.results:
            return defer.succeed(None)
        return defer.succeed(self._fill_extra_data(test_resultid, self.results[test_resultid]))

    # returns a Deferred
    def getTestResults(self, builderid, test_result_setid, result_spec=None):
        ret = []
        for id, row in sorted(self.results.items()):
            if row['builderid'] != builderid:
                continue
            if row['test_result_setid'] != test_result_setid:
                continue
            ret.append(self._fill_extra_data(id, row))

        if result_spec is not None:
            ret = self.applyResultSpec(ret, result_spec)

        return defer.succeed(ret)
